@@ -1,4 +1,4 @@
-/* GTK+ - accessibility implementations
+/* GAIL - The GNOME Accessibility Implementation Library
  * Copyright 2001 Sun Microsystems Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -18,15 +18,11 @@
 #ifndef __GTK_PROGRESS_BAR_ACCESSIBLE_H__
 #define __GTK_PROGRESS_BAR_ACCESSIBLE_H__
 
-#if !defined (__GTK_A11Y_H_INSIDE__) && !defined (GTK_COMPILATION)
-#error "Only <gtk/gtk-a11y.h> can be included directly."
-#endif
-
-#include <gtk/a11y/gtkwidgetaccessible.h>
+#include "gtkwidgetaccessible.h"
 
 G_BEGIN_DECLS
 
-#define GTK_TYPE_PROGRESS_BAR_ACCESSIBLE                      (gtk_progress_bar_accessible_get_type ())
+#define GTK_TYPE_PROGRESS_BAR_ACCESSIBLE                      (_gtk_progress_bar_accessible_get_type ())
 #define GTK_PROGRESS_BAR_ACCESSIBLE(obj)                      (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_PROGRESS_BAR_ACCESSIBLE, GtkProgressBarAccessible))
 #define GTK_PROGRESS_BAR_ACCESSIBLE_CLASS(klass)                        (G_TYPE_CHECK_CLASS_CAST ((klass), GTK_TYPE_PROGRESS_BAR_ACCESSIBLE, GtkProgressBarAccessibleClass))
 #define GTK_IS_PROGRESS_BAR_ACCESSIBLE(obj)                       (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_PROGRESS_BAR_ACCESSIBLE))
@@ -49,8 +45,7 @@ struct _GtkProgressBarAccessibleClass
   GtkWidgetAccessibleClass parent_class;
 };
 
-GDK_AVAILABLE_IN_ALL
-GType gtk_progress_bar_accessible_get_type (void);
+GType _gtk_progress_bar_accessible_get_type (void);
 
 G_END_DECLS
 
