@@ -15,23 +15,22 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GTK_GRADIENT_H__
-#define __GTK_GRADIENT_H__
-
 #if !defined (__GTK_H_INSIDE__) && !defined (GTK_COMPILATION)
 #error "Only <gtk/gtk.h> can be included directly."
 #endif
 
+#ifndef __GTK_GRADIENT_H__
+#define __GTK_GRADIENT_H__
+
 #include <gdk/gdk.h>
 #include <gtk/gtkstylecontext.h>
-#include <gtk/deprecated/gtkstyleproperties.h>
+#include <gtk/gtkstyleproperties.h>
 #include <gtk/deprecated/gtksymboliccolor.h>
 
 G_BEGIN_DECLS
 
 #define GTK_TYPE_GRADIENT (gtk_gradient_get_type ())
 
-GDK_DEPRECATED_IN_3_8
 GType         gtk_gradient_get_type       (void) G_GNUC_CONST;
 
 GDK_DEPRECATED_IN_3_8
