@@ -20,7 +20,6 @@
 #include "config.h"
 
 #include "gtkiconhelperprivate.h"
-#include "string.h"
 
 G_DEFINE_TYPE (GtkIconHelper, _gtk_icon_helper, G_TYPE_OBJECT)
 
@@ -326,7 +325,10 @@ _gtk_icon_helper_ensure_pixbuf (GtkIconHelper *self,
 
     case GTK_IMAGE_STOCK:
       icon_set = gtk_style_context_lookup_icon_set (context, self->priv->stock_id);
-      ensure_pixbuf_for_icon_set (self, context, icon_set);
+      if (icon_set != NULL)
+	ensure_pixbuf_for_icon_set (self, context, icon_set);
+      else
+	pixbuf = NULL;
       break;
 
     case GTK_IMAGE_ICON_SET:
@@ -467,8 +469,7 @@ _gtk_icon_helper_set_stock_id (GtkIconHelper *self,
 {
   _gtk_icon_helper_clear (self);
 
-  if (stock_id != NULL &&
-      g_strcmp0 (stock_id, "") != 0)
+  if (stock_id != NULL)
     {
       self->priv->storage_type = GTK_IMAGE_STOCK;
       self->priv->stock_id = g_strdup (stock_id);
